@@ -102,10 +102,6 @@ module AtCoder.Extra.Seq
 
     -- * Handle (re-exports)
     Handle (..),
-    -- TODO: hide
-    newHandle,
-    nullHandle,
-    invalidateHandle,
 
     -- * Re-exports
     SegAct (..),
@@ -173,7 +169,7 @@ module AtCoder.Extra.Seq
   )
 where
 
-import AtCoder.Extra.Pool (Handle (..), invalidateHandle, newHandle, nullHandle)
+import AtCoder.Extra.Pool (Handle (..), newHandle)
 import AtCoder.Extra.Pool qualified as P
 import AtCoder.Extra.Seq.Raw (Seq (..))
 import AtCoder.Extra.Seq.Raw qualified as Seq
@@ -287,7 +283,7 @@ merge4 seq (Handle hA) (Handle hB) (Handle hC) (Handle hD) = stToPrim $ do
   a <- VGM.unsafeRead hA 0
   b <- VGM.unsafeRead hB 0
   c <- VGM.unsafeRead hC 0
-  d <- VGM.unsafeRead hC 0
+  d <- VGM.unsafeRead hD 0
   root' <- Seq.merge4ST seq a b c d
   VGM.unsafeWrite hA 0 root'
   VGM.unsafeWrite hB 0 P.undefIndex
@@ -401,7 +397,8 @@ write seq (Handle hRoot) k v = stToPrim $ do
     )
     0
 
--- | Amortized \(O(\log n)\). Modifies the \(k\)-th node's monoid value.
+-- | Amortized \(O(\log n)\). Given a user function \(f\), modifies the \(k\)-th node's monoid value
+-- with it.
 --
 -- ==== Constraints
 -- - \(0 \le k \lt n\)
